#!/usr/bin/perl -w
#
# $Id: addalbum.pl,v 1.4 2002/02/06 11:59:58 dom Exp $
# Copyright (C) 2001, 2002 Dominic Hargreaves <dom@earth.li>
# Takes a list of the mp3 ids on the command line and updates the media.albums
# database accordingly.
#
# This file is part of mediadbs.
# 
# mediadbs is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# mediadbs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with mediadbs; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use DBI();
use Getopt::Std;

# Read in options from config file
require "config.pl";
use vars qw($dbhost $dbname $dbuser $dbpass);

sub usage;
sub time_format;
my %options;
my $debug = undef;
#$debug = "foo";
my $firsttrack = 1;


getopts("v", \%options);

die usage unless $ARGV[0];

# Connect to the database
my $dbh = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost",
                        $dbuser, $dbpass, {'RaiseError' => 1});

# Main loop

my $albumid;

MAINLOOP: for (@ARGV) {
   my ($album, $artist, $year, $genre, $query_fields, $query_values, $created);

   ### First song only.
   if ($firsttrack == 1) {
      my $sth = $dbh->prepare("SELECT id, artist, album, year, genre FROM mp3 WHERE id=\"$_\"");
      $sth->execute();
      while (my $mp3ref = $sth->fetchrow_hashref()) {
         die "File with id $_ doesn't exist in the mp3 database" unless $mp3ref->{'id'};
         $query_fields = "title, artist, year, genre, firsttrackid";
         if ($options{v}) {
            $artist = "Various";
         } else {
            $artist = $mp3ref->{'artist'};
         }
         $query_values = "\"$mp3ref->{'album'}\", \"$artist\", \"$mp3ref->{'year'}\", \"$mp3ref->{'genre'}\", \"$_\"";
      }
      $dbh->do("INSERT INTO albums ($query_fields) VALUES ($query_values)");

      # Get albumid back
      $sth = $dbh->prepare("SELECT albumid FROM albums WHERE firsttrackid=\"$_\"");
      $sth->execute();
      while (my $mp3ref = $sth->fetchrow_hashref()) {
         if ($mp3ref->{'albumid'}) {
            $albumid = $mp3ref->{'albumid'};
         } else {
            die "Missing albumid on album we just created!";
         }
      }
      $firsttrack = 0;
   }

   ###

   my $sth = $dbh->prepare("SELECT id, albumid FROM albumjoins WHERE id=\"$_\"");
   $sth->execute();

   # Check whether the file is already in the album database.
   EXISTCHECK: while (my $mp3ref = $sth->fetchrow_hashref()) {
      last unless $mp3ref->{'id'};
      die "$_ already exists in the album db with albumid $mp3ref->{'albumid'}\n";
   }

   # Do the deed.
   $query_fields = "id, albumid";
   $query_values = "\"$_\", \"$albumid\"";
   $dbh->do("INSERT INTO albumjoins ($query_fields) VALUES ($query_values)");
   $sth = $dbh->prepare("SELECT updated FROM albums WHERE albumid=\"$albumid\"");
   $sth->execute();
   while (my $mp3ref = $sth->fetchrow_hashref()) {
      $created = $mp3ref->{'updated'};
      $dbh->do("UPDATE albums SET created=? WHERE albumid=\"$albumid\"",
      undef, $created);
   }
}

# Clean up

$dbh->disconnect();

sub usage {
   print<<EOF;
Syntax: $0 [-v] <list of file IDs from media.mp3>
-v will cause the artist to be set to "Various", instead of the artist of the
first file.
EOF
   exit 1;
}
