#!/usr/bin/perl -w
#
# $Id: purge.pl,v 1.7 2002/02/06 11:59:58 dom Exp $
# Copyright (C) 2001, 2002 Dominic Hargreaves <dom@earth.li>
# Removes files that don't exist from the database.
#
# This file is part of mediadbs.
# 
# mediadbs is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# mediadbs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with mediadbs; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use DBI();
use Getopt::Std;

require "config.pl";
use vars qw($dbhost $dbname $dbuser $dbpass);

sub usage;
my %options;
my $file;

getopts("r:s:", \%options);
$options{s} and $options{r} or usage();

# Connect to the database
my $dbh = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost",
                        $dbuser, $dbpass, {'RaiseError' => 1});

# Main loop

my $sth = $dbh->prepare("SELECT * FROM mp3 WHERE server=\"$options{s}\"");
$sth->execute();
while (my $ref = $sth->fetchrow_hashref()) {
   $file = "$options{r}/$ref->{'path'}/$ref->{'filename'}";

   if (! -f $file) {
      # Paranoia check to make sure the problem isn't elsewhere
      # This isn't actually very useful - better to add on the outermost level
      # of $ref->{'path'}.
      die "Directory $options{r} not readable, aborting." if (! -r $options{r});
      print "Deleting $file ($ref->{'id'}) from database\n";
      $dbh->do("DELETE from mp3 WHERE id=\"$ref->{'id'}\"");
   }
}

# Clean up

$dbh->disconnect();

sub usage {
   print <<EOF;
Syntax: $0 -s [server] -r [root]
Here it's important that the directory is readable (ie properly mounted, etc)
otherwise data will be erroneously removed from the database (though it will
be recreated the next time the update script is run.)
EOF
   exit 1;
}
