#!/usr/bin/perl -w
#
# $Id: updatedb.pl,v 1.10 2002/02/06 11:59:58 dom Exp $
# Copyright (C) 2001, 2002 Dominic Hargreaves <dom@earth.li>
# Takes a list of the full paths of mp3s on STDIN and updated the media.mp3
# database accordingly.
#
# This file is part of mediadbs.
# 
# mediadbs is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# mediadbs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with mediadbs; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use DBI();
use MP3::Info;
use Getopt::Std;
use_winamp_genres();

# Read in options from config file
require "config.pl";
use vars qw($dbhost $dbname $dbuser $dbpass);

sub usage;
sub time_format;
my %options;
my $debug = undef;
#$debug = "foo";

getopts("fs:r:", \%options);
$options{s} && $options{r} or usage();

# Connect to the database
my $dbh = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost",
                        $dbuser, $dbpass, {'RaiseError' => 1});

# Main loop

MAINLOOP: while(<STDIN>) {
   my $old_id = undef;
   my ($file_updated, $db_updated, $created, $info, $tag, $filename, $path);
   my ($genreid, $vbr, $tracknum);
   my ($query_fields_1, $query_fields_2, $query_fields);
   my ($query_values_1, $query_values_2, $query_values);
   chomp;
   print "found file: $_\n" if $debug;
   next MAINLOOP unless (-f $_);
   my $splitfile = $_;
   $splitfile =~ s#$options{r}/##;
   my @components = split /\//, $splitfile;
   $filename = pop @components;
   $path = join "/", @components;
   my $sth = $dbh->prepare("SELECT id, updated FROM mp3 WHERE filename=\"$filename\" AND
    path=\"$path\" AND server=\"$options{s}\"");
   $sth->execute();

   # Check whether the file is already in the database.
   EXISTCHECK: while (my $mp3ref = $sth->fetchrow_hashref()) {
      last unless $mp3ref->{'id'};
      $db_updated = $mp3ref->{'updated'};
      print "$_ already exists in the db with id $mp3ref->{'id'}\n" if $debug;
      my @foo = stat($_);
      $file_updated = time_format($foo[9]);
      if (($db_updated > $file_updated) or (-l $_) or !($options{f})) {
         print "Not updating\n" if $debug;
         next MAINLOOP;
      } else {
         print "Updating entry for $_\n";
         $old_id = $mp3ref->{'id'};
         $dbh->do("DELETE from mp3 WHERE id=\"$mp3ref->{'id'}\"");
         last EXISTCHECK;
      }
   }

   # Extract and add file data.
   print "going to find out about file $_\n" if $debug;
   $info = get_mp3info($_);
   if (!(defined $info->{BITRATE})) {
      print "$_ does not seem to be a valid mp3 file; skipping\n";
      next MAINLOOP;
   }
   if ($info->{VBR}) {
      $vbr = "1";
   } else {
      $vbr = "0";
   }
   $query_fields_1 = "server, path, filename, bitrate, version, mm, ss, stereo, layer, mode, copyright, frequency, size, frames, frame_length, vbr";
   $query_fields_2 = "title, artist, year, album, genre, comment, tracknum";
   $query_values_1 = "\"$options{s}\", \"$path\", \"$filename\", $info->{BITRATE}, $info->{VERSION}, $info->{MM}, $info->{SS}, $info->{STEREO}, $info->{LAYER}, $info->{MODE}, $info->{COPYRIGHT}, $info->{FREQUENCY}, $info->{SIZE}, $info->{FRAMES}, $info->{FRAME_LENGTH}, $vbr";
   if ($tag = get_mp3tag($_)) {
      print "$_ has an id3 tag\n";
      if ($tag->{GENRE}) {
         if ($tag->{GENRE} eq "default") {
            $genreid = 0;
         } else {
            my $sth = $dbh->prepare("SELECT id FROM genre WHERE genre=\"$tag->{GENRE}\"");
            $sth->execute();
            while (my $mp3ref = $sth->fetchrow_hashref()) {
               $genreid = $mp3ref->{'id'};
            }
         }
      } else {
         $genreid = "255";
      }
      if ($tag->{TRACKNUM}) {
         $tracknum = $tag->{TRACKNUM};
      } else {
         $tracknum = "";
      }
      for (qw(TITLE ARTIST ALBUM COMMENT)) {
         $tag->{$_} =~ s/"/\\"/g;
         $tag->{$_} =~ s/'/\\'/g;
      }
      $query_values_2 = "\"$tag->{TITLE}\", \"$tag->{ARTIST}\", \"$tag->{YEAR}\", \"$tag->{ALBUM}\", \"$genreid\", \"$tag->{COMMENT}\", \"$tracknum\"";
      $query_fields = $query_fields_1 . ", " . $query_fields_2;
      $query_values = $query_values_1 . ", " . $query_values_2;
      if ($old_id) {
         $query_fields = "id, " . $query_fields;
         $query_values = $old_id . ", " . $query_values;
      }
   } else {
      print "$_ doesn't have id3\n";
      $query_fields = $query_fields_1;
      $query_values = $query_values_1;
      if ($old_id) {
         $query_fields = "id, " . $query_fields;
         $query_values = $old_id . ", " . $query_values;
      }
   }

   # Turn on db debug output
#   $dbh->trace("2");

   # Do the deed.
   $dbh->do("INSERT INTO mp3 ($query_fields) VALUES ($query_values)");
   $sth = $dbh->prepare("SELECT updated FROM mp3 WHERE filename=\"$filename\" AND
    path=\"$path\" AND server=\"$options{s}\"");
   $sth->execute();
   while (my $mp3ref = $sth->fetchrow_hashref()) {
      $created = $mp3ref->{'updated'};
      $dbh->do("UPDATE mp3 SET created=? WHERE filename=\"$filename\" AND
                 path=\"$path\" AND server=\"$options{s}\"",
      undef, $created);
   }
   print "Adding file $_\n";
}

# Clean up

$dbh->disconnect();

sub usage {
   print<<EOF;
Syntax: $0 -s [server] -r [server path] {-f}
This accepts a list of files with full local paths of mp3s on STDIN
[server] is the name of the server, in lowercase.
[server path] is the local path of the server, such that
//tirian/ maps to /mnt, and //callisto/ maps to /mnt/callisto.
Note this is important; you must use /mnt and not /mnt/ otherwise things
will go wrong. This needs fixing.
-f forces an update for all files - otherwise it will only update a record
if the file's timestamp is newer than the entry
EOF
   exit 1;
}

sub time_format {
   my ($sec, $min,$hour,$day,$mon,$year) = (gmtime($_[0]))[0,1,2,3,4,5];

   $mon += 1;
   $year += 1900;
   $mon = "0" . $mon if $mon < 10;
   $hour = "0" . $hour if $hour < 10;
   $day = "0" . $day if $day < 10;
   $min = "0" . $min if $min < 10;
   $sec = "0" . $sec if $sec < 10;   
   my $datestr=$year . $mon . $day . $hour . $min . $sec;
   $datestr;
}
